
rm(list=ls())

# Set working directory to source file location
# setwd("~/Dropbox/chun-stuff/orm/quasi-reproducible")

## Loading simulated data on ART initiators and baseline covariates.  This was created in 
##  the code for the long-term mortality study (Carriquiry, et al., 2015)
load("d-second-chunk.Rda")

## Loading simulated repeated CD4 measurements
d.cd4<-read.csv("lab_cd4_sim2.csv")

## Removing records with missing or undetectable baseline viral load and missing CD4
d<-d[!is.na(d$rna1),]
d<-d[d$rna1>0,]
d<-d[!is.na(d$nadir.cd4.prehaart),]

##  Merging 6-month CD4 measurements with baseline data
d.ids<-d[,c("id","init.start.date")]
ids<-d.ids$id
d.cd4.new<-d.cd4[d.cd4$patient %in% ids,]
d.cd4.new1<-d.cd4.new[d.cd4.new$time<300 & d.cd4.new$time>60,]
d.cd4.new2<-d.cd4.new1[!duplicated(d.cd4.new1$patient),]
d.cd4.new2$post.cd4<-d.cd4.new2$cd4_v
d.cd4.new3<-d.cd4.new2[,c("patient","post.cd4")]

d1<-merge(d.cd4.new3, d, by.x="patient", by.y="id", all.x=TRUE)

##  Re-naming variables and creating a new dataset that mimics the one used in the CPM CD4 analysis
d1$nadir.cd4<-d1$nadir.cd4.prehaart
d1$init.year<-d1$year.init
d1$pre.rna<-d1$rna1

d<-d1[,c("site","male","age","init.class","route","nadir.cd4","init.year","pre.rna","preARTAIDS.clinical","post.cd4")]


#######################################################################################
##########################  Code for CPMs    ##########################################
#######################################################################################


################################### FUNCTIONS ################

#### function to estimate conditional mean and its standard error for orm model
mean.orm <- function(mod, new.data, se=TRUE){
  if(is.null(mod$yunique)) {
    stop("Need to set x=TRUE and y=TRUE for orm") 
  } else{
    order.y <- mod$yunique
    n.alpha <- length(order.y)-1
    xb <- as.matrix(new.data)%*%matrix(coef(mod)[colnames(new.data)])
    m.alpha <- mod$coef[1:n.alpha]
    lb <- t(outer(m.alpha, xb, "+")[,,1])
    m.s <- mod$trans$cumprob(lb)
    m.f <- t(apply(m.s, 1, FUN=function(x) c(1,x[1:n.alpha]) - c(x[1:n.alpha], 0)))
    m.mean <- apply(m.f, 1, FUN=function(x) sum(x*order.y))
    
    if(se){
      if(mod$family=="logistic") mod$trans$deriv <- function(x) exp(-x)/(1+exp(-x))^2
      
      dmean.dalpha <- t(apply(mod$trans$deriv(lb), 1, FUN=function(x) x*(order.y[2:length(order.y)] - order.y[1:n.alpha])))
      dmean.dbeta <- apply(dmean.dalpha, 1, sum)*as.matrix(new.data)
      dmean.dtheta <- cbind(dmean.dalpha, dmean.dbeta)   
      mean.var <-diag(dmean.dtheta%*%solve(mod$info.matrix)%*%t(dmean.dtheta))
      mean.se <- sqrt(mean.var)   
      result <- cbind(m.mean, mean.se)
      ci <- t(apply(result, 1, FUN=function(x) c(x[1]- qnorm(0.975)*x[2], x[1]+ qnorm(0.975)*x[2])))
      result <- cbind(result, ci)
      colnames(result) <- c("est", "se", "lb", "ub")
    } else{
      result <- matrix(m.mean)
      colnames(result) <- c("est")
    }
    
    
    return(result)
    
    
  } 
  
}


#### function to estimate conditional quantiles and confidence intervals for orm model
quantile.orm <- function(mod, new.data, probs=0.5, se=TRUE){
  
  quantile <- matrix(NA, nrow=dim(new.data)[1], ncol=length(probs))
  order.y <- mod$yunique
  #n.alpha <- length(order.y)-1
  xb <- as.matrix(new.data)%*%matrix(coef(mod)[colnames(new.data)])
  alpha <- mod$coef[1:(length(unique(order.y))-1)]
  lb <- t(outer(alpha, xb, "+")[,,1])
  m.cdf <- 1- mod$trans$cumprob(lb)
  m.cdf <- cbind(0, m.cdf, 1)
  for(i in 1: length(probs)){
    try({
      index.1 <- apply(m.cdf, 1, FUN=function(x){ max(which(x<=probs[i]))[1]} )
      index.2 <- apply(m.cdf, 1, FUN=function(x){ min(which(x>=probs[i]))[1]} )
      
      index.y1 <- ifelse(index.1>length(order.y), Inf, order.y[index.1])
      index.y2 <- ifelse(index.2>length(order.y),Inf,order.y[index.2])
      
      index.y1.cdf <- ifelse(index.1==0, 0, m.cdf[cbind(1:dim(new.data)[1], index.1)])
      
      index.y2.cdf <- ifelse(index.2>length(order.y), 1, m.cdf[cbind(1:dim(new.data)[1], index.2)])
      
      
      
      
      quantile[,i] <- ifelse(index.1==index.2, index.y1, 
                             (index.y2-index.y1)/(index.y2.cdf - index.y1.cdf)*(probs[i]-index.y1.cdf) + index.y1) 
      quantile[, i] <- ifelse(is.infinite(quantile[,i]), max(order.y), quantile[, i])
    })
    
  }
  result <- quantile
  
  if(se){
    if(mod$family=="logistic") mod$trans$deriv <- function(x) exp(-x)/(1+exp(-x))^2
    
    quantile.lb <- quantile.ub <- matrix(NA, nrow=dim(new.data)[1], ncol=length(probs))
    lb.se <- matrix(NA, ncol=dim(lb)[2], nrow=dim(new.data)[1])
    var <- as.matrix(solve(mod$info.matrix))
    
    for(i in 1:dim(lb)[2]){
      var.i <- var[c(i, which(names(coef(mod)) %in% colnames(new.data))), 
                   c(i, which(names(coef(mod)) %in% colnames(new.data)))]
      
      dcdf.dtheta <- cbind(-mod$trans$deriv(lb[,i]),  
                           -mod$trans$deriv(lb[,i])*as.matrix(new.data) )
      dlb.dtheta <- as.matrix(cbind(1, new.data))
      lb.se[,i] <- sqrt(diag(dlb.dtheta%*%var.i%*% t(dlb.dtheta)))
    }
    
    ci.lb <- sapply(1:dim(lb)[2], FUN=function(i) { 1- mod$trans$cumprob(lb[, i] +qnorm(0.975)*lb.se[, i])})
    ci.ub <- sapply(1:dim(lb)[2], FUN=function(i) { 1- mod$trans$cumprob(lb[, i] -qnorm(0.975)*lb.se[, i])})
    ci.lb <- matrix(ci.lb, nrow=dim(new.data)[1])
    ci.ub <- matrix(ci.ub, nrow=dim(new.data)[1])
    
    ci.lb <- cbind(0, ci.lb, 1)
    ci.ub <- cbind(0, ci.ub, 1)
    
    for(i in 1: length(probs)){
      try({
        index.1 <- apply(ci.lb, 1, FUN=function(x){ max(which(x<=probs[i]))[1]} )
        index.2 <- apply(ci.lb, 1, FUN=function(x){ min(which(x>=probs[i]))[1]} )
        
        index.y1 <- ifelse(index.1>length(order.y), Inf, order.y[index.1])
        index.y2 <- ifelse(index.2>length(order.y),Inf,order.y[index.2])
        
        index.y1.cdf <- ifelse(index.1==0, 0, ci.lb[cbind(1:dim(new.data)[1], index.1)])
        
        index.y2.cdf <- ifelse(index.2>length(order.y), 1, ci.lb[cbind(1:dim(new.data)[1], index.2)])
        
        
        quantile.lb[,i] <- ifelse(index.1==index.2, index.y1, 
                                  (index.y2-index.y1)/(index.y2.cdf - index.y1.cdf)*(probs[i]-index.y1.cdf) + index.y1) 
        quantile.lb[, i] <- ifelse(is.infinite(quantile.lb[,i]), max(order.y), quantile.lb[, i])
        
        index.1 <- apply(ci.ub, 1, FUN=function(x){ max(which(x<=probs[i]))[1]} )
        index.2 <- apply(ci.ub, 1, FUN=function(x){ min(which(x>=probs[i]))[1]} )
        
        index.y1 <- ifelse(index.1>length(order.y), Inf, order.y[index.1])
        index.y2 <- ifelse(index.2>length(order.y),Inf,order.y[index.2])
        
        index.y1.cdf <- ifelse(index.1==0, 0, ci.ub[cbind(1:dim(new.data)[1], index.1)])
        
        index.y2.cdf <- ifelse(index.2>length(order.y), 1, ci.ub[cbind(1:dim(new.data)[1], index.2)])
        
        
        quantile.ub[,i] <- ifelse(index.1==index.2, index.y1, 
                                  (index.y2-index.y1)/(index.y2.cdf - index.y1.cdf)*(probs[i]-index.y1.cdf) + index.y1) 
        quantile.ub[, i] <- ifelse(is.infinite(quantile.ub[,i]), max(order.y), quantile.ub[, i])
        
        
      })
      
    }
    
    result <- list(quantile=quantile,
                   lb=quantile.ub,
                   ub=quantile.lb)
    
    
    
  }
  
  
  
  return(result)
  
}


#### function to estimate conditional CDF and its standard error for orm models
cdf.orm <- function(mod, new.data, at.y=0,se=TRUE){
  if(is.null(mod$yunique)) {
    stop("Need to set x=TRUE and y=TRUE for orm") 
  } else{
    order.y <- mod$yunique
    xb <- as.matrix(new.data)%*%matrix(coef(mod)[colnames(new.data)])
    
    index <- sapply(at.y, FUN=function(x) {if(x<min(order.y)[1]) result <- Inf 
    else if (x==min(order.y)[1]) result <- 1
    else if(x >= max(order.y)[1]) result <- -Inf
    else which(order.y>=x)[1]-1})
    
    m.alpha <- mod$coef[index]
    m.alpha <- ifelse(is.infinite(index), index, m.alpha)
    if(length(at.y)==1){
      lb <- as.matrix(outer(m.alpha, xb, "+")[,,1])
    } else lb <- t(outer(m.alpha, xb, "+")[,,1])
    m.cdf <- 1- mod$trans$cumprob(lb)
    
    
    if(se){
      if(mod$family=="logistic") mod$trans$deriv <- function(x) exp(-x)/(1+exp(-x))^2
      cdf.se <- matrix(NA, ncol=length(at.y), nrow=dim(new.data)[1])
      lb.se <- matrix(NA, ncol=length(at.y), nrow=dim(new.data)[1])
      
      var <- as.matrix(solve(mod$info.matrix))
      
      for(i in 1:length(at.y)) {
        
        var.i <- var[c(index[i], which(names(coef(mod)) %in% colnames(new.data))), 
                     c(index[i], which(names(coef(mod)) %in% colnames(new.data)))]
        dcdf.dtheta <- cbind(-mod$trans$deriv(lb[,i]),  
                             -mod$trans$deriv(lb[,i])*as.matrix(new.data) )
        dlb.dtheta <- as.matrix(cbind(1, new.data))
        cdf.se[,i] <- sqrt(diag(dcdf.dtheta %*% var.i%*% t(dcdf.dtheta)))
        lb.se[, i] <- sqrt(diag(dlb.dtheta%*%var.i%*% t(dlb.dtheta)))
        
      }
      ci.lb <- sapply(1:length(at.y), FUN=function(i) { 1- mod$trans$cumprob(lb[, i] +qnorm(0.975)*lb.se[, i])})
      ci.ub <- sapply(1:length(at.y), FUN=function(i) { 1- mod$trans$cumprob(lb[, i] -qnorm(0.975)*lb.se[, i])})
      
      
      result <- list(est=m.cdf,
                     se=cdf.se,
                     lb=ci.lb,
                     ub=ci.ub)
    } else{
      result <- list(est=m.cdf)
    }
    
    
    return(result)
    
  } 
  
}


##### function for calculate probility-scale residuals. 
########This has also been implemented as presid() in "PResiduals" package
presid.orm <- function(object){
  k <- object$non.slopes
  L <- object$linear.predictors
  trans <- object$trans
  cumprob <- if(length(trans)) trans$cumprob else plogis
  if(length(L)==0) stop('you did not use linear.predictors=TRUE for the fit')
  if(length(Y <- object$y) == 0) stop("you did not specify y=TRUE in the fit")
  if(!is.factor(Y)) Y <- factor(Y)
  Y <- unclass(Y) - 1
  cof <- object$coefficients
  if(length(X <- object$x)==0)
    stop("you did not use x=TRUE in the fit")
  
  
  N <- length(Y)
  px <- 1 - cumprob(outer(cof[1:k],
                          as.vector(X %*% cof[- (1:k)]), "+"))
  low.x = rbind(0, px)[cbind(Y + 1L, 1:N)]
  hi.x  = 1 - rbind(px, 1)[cbind(Y + 1L, 1:N)]
  return(low.x - hi.x)
}

#### Or use presid(), see details in the help documents
library(PResiduals)
library(rms)
help(presid)


###### cumulative probability models with different link functions #####
dd<-datadist(d); options(datadist='dd')
orm.logit <- orm(post.cd4~ site + male + rcs(age) + init.class + route + 
                   rcs(sqrt(nadir.cd4)) +rcs(init.year)+ rcs(log(pre.rna))+
                   preARTAIDS.clinical, data=d, x=TRUE, y=TRUE)

orm.probit <- orm(post.cd4~ site + male + rcs(age) + init.class + route + 
                    rcs(sqrt(nadir.cd4)) +rcs(init.year)+ rcs(log(pre.rna))+
                    preARTAIDS.clinical, data=d, x=TRUE, y=TRUE, family=probit)

orm.cloglog <- orm(post.cd4~ site + male + rcs(age) + init.class + route + 
                     rcs(sqrt(nadir.cd4)) +rcs(init.year)+ rcs(log(pre.rna))+
                     preARTAIDS.clinical, data=d, x=TRUE, y=TRUE, family=cloglog)

orm.loglog <- orm(post.cd4~ site + male + rcs(age) + init.class + route + 
                    rcs(sqrt(nadir.cd4)) +rcs(init.year)+ rcs(log(pre.rna))+
                    preARTAIDS.clinical, data=d, x=TRUE, y=TRUE, family=loglog)### loglog in orm correspoonds to cloglog in conventional cumulative link models

log.L <- c(-orm.probit$deviance[2], -orm.logit$deviance[2], -orm.cloglog$deviance[2], -orm.loglog$deviance[2])/2

names(log.L) <- c("probit", "logit", "cloglog", "loglog")
log.L


presid.logit <- presid.orm(orm.logit)

presid.probit <- presid.orm(orm.probit)

presid.cloglog <- presid.orm(orm.cloglog)


###  Creating a new dataset where all variables are set to representative levels, 
###  except age which varies from 18 to 82 years

new.age <- (180:820)/10
new.data <- matrix(NA, ncol=dim(orm.logit$x)[2], nrow=length(new.age))

colnames(new.data) <- colnames(orm.logit$x)
for(i in 1:5) new.data[,i] <- rep(orm.logit$x[which(d$site=="brazil")[1], i], length(new.age))

new.data[, "male=1"] <- 1
new.data[, 7:10] <-  rcspline.eval(new.age, knots=orm.logit$Design$parms$age, inclx=TRUE)

for(i in 11:13) new.data[,i] <- rep(orm.logit$x[which(d$init.class=="NNRTI")[1], i], length(new.age))
for(i in 14:17) new.data[,i] <- rep(orm.logit$x[which(d$route=="homo/bisexual")[1], i], length(new.age))

for(i in 18:21) new.data[,i] <- rep(orm.logit$x[which(d$nadir.cd4==167)[1], i], length(new.age))
for(i in 22:25) new.data[,i] <- rep(orm.logit$x[which(d$init.year==2010)[1], i], length(new.age))
new.data[,26:29] <- matrix(rcspline.eval(log(91728),  knots=orm.logit$Design$parms$pre.rna, inclx=TRUE),nrow=length(new.age), ncol=4, byrow=TRUE)
for(i in 30:31) new.data[,i] <- rep(orm.logit$x[which(d$preARTAIDS.clinical=="not AIDS")[1], i], length(new.age)) 


### Estimated mean as a function of age
est.mean.orm.logit <- mean.orm(orm.logit,new.data,se=TRUE )

par(mfrow=c(1,1))
plot(c(new.age,new.age),c(est.mean.orm.logit[,3],est.mean.orm.logit[,4]),
     type="n",xlab="Age (years)", ylab="Expected CD4")
lines(new.age,est.mean.orm.logit[,1])
lines(new.age,est.mean.orm.logit[,3],lty=2)
lines(new.age,est.mean.orm.logit[,4],lty=2)


### Estimated median as a function of age
est.median.orm.logit <- quantile.orm(mod=orm.logit,new.data=new.data, probs=0.5,se=TRUE )
names(est.median.orm.logit)

plot(c(new.age, rev(new.age)), c(est.median.orm.logit$lb, rev(est.median.orm.logit$ub)), ylab="Median CD4", xlab="Age (years)", type="n")
polygon(c(new.age, rev(new.age)), c(est.median.orm.logit$lb, rev(est.median.orm.logit$ub)), col = 'grey', border = NA)
lines(new.age, est.median.orm.logit$quantile, cex=0.01)
lines(new.age, est.median.orm.logit$lb, cex=0.01, lty=2)
lines(new.age, est.median.orm.logit$ub, cex=0.01, lty=2)


### Estimated CDF as a function of age
est.cdf.orm.logit <- cdf.orm(mod=orm.logit, new.data=new.data, at.y=350)
names(est.cdf.orm.logit)

plot(c(new.age,new.age),c(est.cdf.orm.logit$lb,est.cdf.orm.logit$ub), cex=0.01, xlab="age", ylab="P(CD4<=350)", typ="n")
polygon(c(new.age, rev(new.age)), c(est.cdf.orm.logit$ub, rev(est.cdf.orm.logit$lb)), col = 'grey', border = NA)
lines(new.age, est.cdf.orm.logit$est, cex=0.01)
lines(new.age, est.cdf.orm.logit$ub, cex=0.01, lty=2)
lines(new.age, est.cdf.orm.logit$lb, cex=0.01, lty=2)


### Wilcoxon rank sum

wilcox.test(post.cd4~male, data=d)
orm(post.cd4~ male, data=d, x=TRUE, y=TRUE)

wilcox.test(post.cd4~preARTAIDS.clinical, subset=preARTAIDS.clinical!="Unknown", data=d)
orm(post.cd4~preARTAIDS.clinical, data=d, subset=preARTAIDS.clinical!="Unknown", x=TRUE, y=TRUE)


###  Probabilistic Index P(Y_1>Y_2|X_1, X_2)

### Only works for logistic link right now
prob.index.orm <- function(mod, new.data, se=TRUE){
  if(is.null(mod$yunique)) {
    stop("Need to set x=TRUE and y=TRUE for orm") 
  } else{
    order.y <- mod$yunique
    n.alpha <- length(order.y)-1
    xb <- as.matrix(new.data)%*%matrix(coef(mod)[colnames(new.data)])
    z<- xb-xb[1]
    p.index<-exp(z)*(exp(z) - z - 1)/(exp(z)-1)^2
  }
  return(p.index)
}
    
p.index<-prob.index.orm(orm.logit, new.data=new.data)
    
plot(new.age, p.index, xlab="Age (years)", ylab="Pr(CD4_1 > CD4_2 | Age_1, Age_2=18")

    
